// CommandPool class implementation
#include "pch.h"

#include "CommandPool.h"
#include "util.h"

namespace nf::client::render {
    CommandPool::CommandPool(const VkDevice& device, const VkQueue& queue, uint32_t queueFamilyIndex)
        : GraphicsResource(device)
        , m_queue(queue)
        , m_handle()
    {
        VkCommandPoolCreateInfo commandPoolCI = { VK_STRUCTURE_TYPE_COMMAND_POOL_CREATE_INFO };
        commandPoolCI.flags = VK_COMMAND_POOL_CREATE_RESET_COMMAND_BUFFER_BIT;
        commandPoolCI.queueFamilyIndex = queueFamilyIndex;

        if (vkCreateCommandPool(m_device, &commandPoolCI, nullptr, &m_handle) != VK_SUCCESS)
            NFError("Could not create command pool.");
    }

    VkCommandBuffer CommandPool::allocateCommandBuffer() const {
        VkCommandBuffer commandBuffer = nullptr;

        VkCommandBufferAllocateInfo commandBufferAI = { VK_STRUCTURE_TYPE_COMMAND_BUFFER_ALLOCATE_INFO };
        commandBufferAI.commandPool = m_handle;
        commandBufferAI.commandBufferCount = 1;

        if (vkAllocateCommandBuffers(m_device, &commandBufferAI, &commandBuffer) != VK_SUCCESS)
            NFError("Could not create command buffer.");

        return commandBuffer;
    }

    VkCommandBuffer CommandPool::beginOneTimeExecution() const {
        VkCommandBuffer commandBuffer = allocateCommandBuffer();

        VkCommandBufferBeginInfo beginInfo = { VK_STRUCTURE_TYPE_COMMAND_BUFFER_BEGIN_INFO };
        beginInfo.flags = VK_COMMAND_BUFFER_USAGE_ONE_TIME_SUBMIT_BIT;

        vkBeginCommandBuffer(commandBuffer, &beginInfo);

        return commandBuffer;
    }

    void CommandPool::endOneTimeExecution(VkCommandBuffer commandBuffer) const {
        vkEndCommandBuffer(commandBuffer);

        VkSubmitInfo submitInfo = { VK_STRUCTURE_TYPE_SUBMIT_INFO };
        submitInfo.commandBufferCount = 1;
        submitInfo.pCommandBuffers = &commandBuffer;

        vkQueueSubmit(m_queue, 1, &submitInfo, nullptr);
        vkQueueWaitIdle(m_queue);

        vkFreeCommandBuffers(m_device, m_handle, 1, &commandBuffer);
    }

    CommandPool::~CommandPool() {
        vkDestroyCommandPool(m_device, m_handle, nullptr);
    }
}
